#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Configure routines and option classes                                         #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: configure.rb,v 1.3 2004/04/08 09:01:40 leikind Exp $                     #
#*******************************************************************************#


module Core

    class Option

        # general option description
        attr_reader :descr, :name

        # the option is on or off
        attr_accessor :on


        def initialize(name, descr)
            @name, @descr = name, descr
            @on = false
        end

        # for debugging purposes
        def to_str
            self.class.to_s.sub("Core::","") + ") " + self.to_s + " : " + @descr
        end

        def body
            "--" + @name
        end

        def param
            ""
        end
    end

    class BoolOption < Option
        def to_s
            if on()
                body()
            else
                ""
            end
        end
    end


    class ParamOption < Option

        attr_accessor :param

        def initialize(name, descr, param_type)
            super(name, descr)
            @param_type = param_type
        end

        def param_filled
            not (@param.nil? or @param =~ /^\s*$/)
        end

        def to_s
            if param_filled() and @on
                body() + "=\"" + @param + "\""
            else
                ""
            end
        end
    end


    class ParamBoolOption < ParamOption

        def param
            if param_filled() and @on
                @param.to_s
            else
                ""
            end
        end


        def to_s
            if on()
                body() + "=\"" + param() +"\""
            else
                ""
            end
        end
    end


    class OptionContainer < Array
        def initialize(*args)
            super(*args)
            @sections = Array.new
        end

        attr_accessor :sections

        def new_section(name)
            @sections << [name, self.length]
        end

        def each_in_section(section_name)
            @sections.each_with_index do|sect, i|
                #puts sect[0] + " " + section_name
                if sect[0] == section_name
                    start = sect[1]
                    finish = (@sections[i+1].class == Array) ? @sections[i+1][1]-1 : self.length-1
                    #finish = 0 if finish == -1
                    #puts @sections[i+1], start, finish
                    #exit
                    self[start..finish].each do |e|
                        yield e
                    end
                end
                #puts i, sect.inspect
            end
        end

        def section_names
            @sections.collect{|name, i| name}.reject{|name| name == "Some influential environment variables"}
        end
    end

    class ConfigureParser
        def initialize(io)
            @io = io
            @oc = OptionContainer.new
        end


        def parse

            while line = @io.gets
                case line
                when /^([A-Z][\w\d_\s-]+):$/
                    if $1 == "Configuration"
                        pass = true
                        next
                    else
                        pass = false
                    end
                    @oc.new_section($1)
                    @last_line_was_an_object = false

                when /--with(out)?-PACKAGE/   # ignore
                    @last_line_was_an_object = false

                when /--(en|dis)able-FEATURE/ # ignore
                    @last_line_was_an_object = false

                when /^\s+([^-\s].*?)\s*$/
                    if @last_line_was_an_object
                        @oc[-1].descr << " " + $1
                    end

                when /^\s+--([\w\d_-]+?)=([\w\d_-]+)(\s+(.+)?\s*)?$/
                    unless pass
                        descr = $4.nil? ? "" : $4
                        @oc << ParamOption.new($1, descr, $2)
                        @last_line_was_an_object = true
                    end

                when /^\s+--([\w\d_-]+?)\[=([\w\d_-]+)\](\s+(.+)?\s*)?$/

                    unless pass
                        descr = $4.nil? ? "" : $4
                        @oc << ParamBoolOption.new($1, descr, $2)
                        @last_line_was_an_object = true
                    end


                when /^\s+--([\w\d_-]+)\s+(.+)\s*/
                    unless pass
                        @oc <<  BoolOption.new($1, $2)
                        @last_line_was_an_object = true
                    end
                else
                    @last_line_was_an_object = false
                end
            end
            return @oc
        end
    end
end

if $0 == __FILE__
File.open("misc/conf1"){|f|
    p = Core::ConfigureParser.new(f)
    @options =  p.parse
}

@options.sections.each do |sect|
    puts "", sect.upcase
    @options.each_in_section(sect) do |e|
    #@options.each do|e|

        case e

        when Core::BoolOption
            e.on = true

        when Core::ParamBoolOption
            e.on = true
            e.param="qqq"
        when Core::ParamOption
            e.param = "qqq"

        end
        puts e.to_str
    end
end
end
