#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Miscellaneous core stuff                                                      #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: core.rb,v 1.17 2004/07/07 15:25:05 leikind Exp $                          #
#*******************************************************************************#


require "core/configure.rb"
require "core/archives.rb"
require "core/sessions.rb"
require "pathname.rb"
require "thread.rb"

module Core

    class StdIOThread < Thread
        def initialize(mutex, io, action)
            super(mutex, io, action){|mutex, io, action|
                while str_err = io.getc
                    char_err = str_err.chr
                    if mutex.locked?
                        if mutex.lock_author == "stdout"
                            mutex.lock
                            mutex.lock_author = "stderr"
                        end
                    else # if there's no lock we'll take it then
                        mutex.lock
                        mutex.lock_author = "stderr"
                    end
                    
                    action.call(char_err)
                    
                    if char_err == "\n" and mutex.locked? and mutex.lock_author == "stderr"
                        mutex.lock_author = nil
                        mutex.unlock
                    end
                end
            }
        end
    end
    
    class MutexWithAuthoredLock < Mutex
        attr_accessor :lock_author
    end
    
    class CommandRun
        def initialize(command, stdout_block, stderr_block)
            @command, @stdout_block, @stderr_block = command, stdout_block, stderr_block
            @mutex = MutexWithAuthoredLock.new
            @stopped = false
        end
        
        def run
            reading, writing = IO.pipe
            @dupped_stderr = STDERR.dup
            STDERR.reopen(writing)
            stdout = IO.popen(@command)
            
            @stderr_listener = StdIOThread.new(@mutex, reading, @stderr_block)
            @stderr_listener.priority = 10

            @stdout_listener = StdIOThread.new(@mutex, stdout, @stdout_block)
            
            @stdout_listener.join
            STDERR.reopen(@dupped_stderr)
            stdout.close
            @stderr_listener.exit
            return @stopped
        end
        
        def stop
            log("stopping")
            @stopped = true
            @stdout_listener.exit
            STDERR.reopen(@dupped_stderr)
            @stderr_listener.exit            
        end
    end

    class IPC
        include Singleton
        require "thread"
        
        def initialize
            @direct_output_listener = nil
        end
        
        def register_direct_output_listener(listener)
            @direct_output_listener = listener
        end

        def stop
            @command.stop
        end

        def system(command, comment)
            notify(UI::Signals::NewCommand, command.external, comment)

            stdout_action = Proc.new{|line|
                send(line, true)
            }

            stderr_action = Proc.new{|line|
                send(line, false)
            }

            @command = CommandRun.new(command.internal, stdout_action, stderr_action)
            res = @command.run
            notify(UI::Signals::CommandFinished)
            return (not res)
        end

        def send(string, stdout)
            #signal = stdout ? UI::Signals::ConsoleLineStdOut : UI::Signals::ConsoleLineStdErr
            #notify(signal, line)
            if stdout
                @direct_output_listener.stdout_listen(string)
            else
                @direct_output_listener.stderr_listen(string)
            end
        end
    end


    class SrcDirectory

        include Singleton
        
        def create
            unless @src_dir.exist?
                Dir.mkdir(@src_dir.to_s)
            end
        end

        DEFAULT_SRC_DIR = ENV["HOME"] + "/easisrc"
        
        def initialize
            @config = UI::EasinstallerApp.get_app.config
            
            if @config["global_src_dir"]
                @src_dir  = Pathname.new(@config["global_src_dir"])
            else
                @src_dir  = Pathname.new(DEFAULT_SRC_DIR)
                @config["global_src_dir"] = DEFAULT_SRC_DIR
            end
    
            create()
        end

        def global_src
            @src_dir
        end

        def global_src=(new_value)
            if new_value.class == String
                new_value = Pathname.new(new_value)
            end
            @src_dir = new_value
            @config["global_src_dir"] = new_value.to_s
            create()
        end


        def get_dir
            n = 1
            while Pathname.new(@src_dir.to_s + "/#{n}").exist?
                n += 1
            end
            res = @src_dir.to_s + "/#{n}"
            Dir.mkdir(res)
            return res
        end
    end


    class Command
        attr_accessor :internal, :external
        alias_method :to_s, :internal

        def initialize(internal, external = nil)
            @internal  = internal
            @external = external.nil? ? @internal : external
        end
    end

end

if $0 == __FILE__

    base  = "/home/leikind/ruby/installer/arch/"

    ["eclipse-JDT-M1.zip",
    "sylpheed-0.9.7claws_.tar",
    "sylpheed-0.9.7claws.tar.bz2",
    "sylpheed-iconset-0.8.3claws.tar.gz"].each do |n|
        Core::Session.new(base + n)
        puts "\n" * 3
     end
end

