#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Sessions                                                                      #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: sessions.rb,v 1.30 2004/07/07 15:25:05 leikind Exp $                      #
#*******************************************************************************#


module Core

    class ActionlessSession
        attr_reader :command_history, :signal_history, :src_dir,
                :last_opened, :first_opened, :options
        attr_accessor :name

        def initialize(filename)
            @filename = filename
            @name = nil
            @command_history = CommandHistory.new
            @signal_history = SignalHistory.new
            @resources = AppToolkit::Resources.instance
            @ipc = Core::IPC.instance
            @valid_for_saving = true
        end

        def src_dir=(dir)
            @src_dir = dir
            if @name.nil?
                @name = File.basename(@src_dir)
            end
        end

        def cleanup()
            FileUtils.rm_rf(@src_dir)

            # deleting parent directories which had been created automatically
            parent = Pathname.new(@src_dir).parent
            if parent.parent ==  Core::SrcDirectory.instance.global_src and parent.basename.to_s =~ /^\d+$/
                parent.delete
            end
        end

        def self.new_from_directory(dir)
            session = ActionlessSession.new(nil)
            session.src_dir =  dir

            session_config_path = session.src_dir + "/.easinstaller"
            if FileTest.exists?(session_config_path)
                session.restore(session_config_path)
            else
                raise "No .easinstaller found in session directory #{session.src_dir}"
            end
            return session
        end

        def restore(path)
            log "restoring actionless"
            File.open(path){|f|
                saved_data = YAML::load(f)
                @name = saved_data["name"]
                @last_opened = saved_data["last opened"]
                @first_opened = saved_data["first opened"]
                @command_history = saved_data["command history"]
                @signal_history = saved_data["signal history"]
                @options = saved_data["options"]
                unless @options.nil?
                    @options.sections = saved_data["sections"]
                end
            }
        end

        def save
            if @valid_for_saving
                log "saving session"
    
                saved_data = {
                    "name" => @name,
                    "last opened" => @last_opened,
                    "first opened" => @first_opened,
                    "command history" => @command_history,
                    "signal history" => @signal_history,
                }
    
                # the session can have no options if the package
                # has only Makefile, no configure or autogen.sh
                unless @options.nil?
                    saved_data["options"] = @options
                    saved_data["sections"] = @options.sections
                end
    
                File.open(@src_dir + "/.easinstaller", "w+"){|f|
                    f.puts saved_data.to_yaml
                }
    
                SessionList.instance.add(self)
            end
        end
    end

    class Session < ActionlessSession

        def initialize(*args)
            super(*args)
            @last_opened = Time.new

            # will always be rewritten in #restore
            @first_opened = Time.new
        end

        def file_present?(filename)
            if FileTest.exists?(@src_dir + "/" + filename)
                true
            else
                false
            end
        end
        

        private :file_present?

        def configure_present?
            file_present?("configure")
        end

        def makefile_present?
            file_present?("Makefile")
        end

        def autogen_present?
            file_present?("autogen.sh")
        end


        def self.new_from_directory(dir)
            session = Session.new(nil)
            session.src_dir =  dir
            return session
        end

        def recognize_filetype
            @archive = Archive.new(@filename)
            if @archive.recognize_filetype
                return true
            else
                return false
            end
        end

        def unpack
            @archive.target_directory = SrcDirectory.instance.get_dir
            res = @archive.unpack
            if res.class == Array
                @valid_for_saving = false
                @src_dir = res[0]
                cleanup()
                return nil        
            else
                self.src_dir = res
            end
        end

        def run(configure_present = true)
            session_config_path = @src_dir + "/.easinstaller"
            if FileTest.exists?(session_config_path)
                restore(session_config_path)
                if @options.nil? # still
                    parse_configure() if configure_present
                end
                @last_opened = Time.new
            else
                parse_configure() if configure_present
            end
        end


        def restore(path)
            log "restoring"
            super(path)
            rerun_signals()
        end

        def rerun_signals
            log "rerun signals"
            @signal_history.logging_off{
                notify(UI::Signals::OptionsParsed, @options) unless @options.nil?
                @signal_history.each do |sig|
                    signal = eval(sig)
                    notify(signal)
                end
            }
        end

        def local_installation?
            if ENV["USER"] == "root"
                return true
            end

            if @options.nil?
                return false
            else
                prefix_option = @options.find{|opt| opt.name == "prefix"}
                if prefix_option.nil?
                    return false
                else
                    value = prefix_option.param
                    if value.nil?
                        return false
                    else
                        if value =~ /^\s*(~|\$PWD)/ or ENV["HOME"] == value[0..ENV["HOME"].length-1]
                            return true
                        else
                            return false
                        end
                    end
                end
            end
        end

        TEMP_FILE = ".easinstaller_temp_file"

        def parse_configure
            Dir.chdir(@src_dir){
                command = Command.new("./configure --help | tee #{TEMP_FILE}", "./configure --help")
                if @ipc.system(command , @resources["running_configure--help"])
                    parser = Core::ConfigureParser.new(File.new(TEMP_FILE))
                    @options =  parser.parse
                    File.unlink(TEMP_FILE)
                    notify(UI::Signals::OptionsParsed, @options)
                end
            }
        end


        def run_autogen
            Dir.chdir(@src_dir){
                command = Command.new("./autogen.sh")
                @ipc.system(command , @resources["running_autogen"])
            }
        end


        def run_configure
            Dir.chdir(@src_dir){
                params = @options.collect{|o| o.to_s}.reject{|s| s == ""}.join(" ")
                command = Command.new("./configure " + params)
                if @ipc.system(command , @resources["running_configure"])
                    if makefile_present?
                        notify(UI::Signals::ConfigureFinished)
                    else
                        notify(UI::Signals::ConfigureFailed)
                    end                    
                end
            }
        end

        def run_build
            Dir.chdir(@src_dir){
                command = Command.new("make")
                if @ipc.system(command , @resources["running_make"])
                    notify(UI::Signals::BuildFinished)
                end
            }
        end

        def run_install(installation_variant)
            Dir.chdir(@src_dir){
                command = nil
                if installation_variant.nil? or installation_variant.class == UI::UserInstallation
                    command = Command.new("make install")
                elsif installation_variant.class == UI::RootInstallation
                    password = installation_variant.password
                    command = Command.new("expect -c 'spawn -noecho su -c \"make install\" ;" +
                        " expect \"Password:\" ; send \"#{password}\\r\" ; interact'", 
                            "make install")
                elsif installation_variant.class == UI::SudoInstallation
                    if installation_variant.using_password?
                        password = installation_variant.password
                        command = Command.new("echo \"#{password}\" | sudo -S make install", 
                            "make install")
                    else
                        command = Command.new("sudo make install", "make install")
                    end
                end
                if @ipc.system(command , @resources["running_make_install"])
                    notify(UI::Signals::InstallFinished)
                end
            }
        end
    end

    class SessionList

        include Singleton
    
        def delete(path)
            @sessions.delete(path)
        end

        def initialize
            config = UI::EasinstallerApp.get_app.config
            @sessions = config["session list"] || Hash.new

            @sessions.each{|path, name|
                unless FileTest.directory?(path)
                    self.delete(path)
                    log "deleting #{path}"
                end
            }
    
            config["session list"] = @sessions
        end

        def get
            @sessions
        end

        def add(session)
            @sessions[session.src_dir] =  File.basename(session.src_dir)
            UI::EasinstallerApp.get_app.config["session list"] = @sessions
            UI::EasinstallerApp.get_app.config.dump
        end
    end

    class CommandHistory < Array
        def initialize
            super()
            hook(UI::Signals::NewCommand){|emitter, command|
                self.push(command)
            }
        end
    end

    class SignalHistory < Array
        include UI::Signals

        @@signals_to_track = [
            #OptionsParsed,
            #RunConfigure,
            ConfigureFinished,
            # RunBuild,
            BuildFinished,
            # RunInstall,
             InstallFinished
            ]

        def logging_off(&block)
            @log_commands = false
            block.call
            @log_commands = true
        end


        def initialize
            super()
            @log_commands = true
            @@signals_to_track.each do |signal|
                hook(signal){
                    self.push(signal.to_s) if @log_commands
                }
            end
        end
    end

end
